import 'package:flutter/services.dart';
import 'package:intl/intl.dart';

class LengthLimitingTextFieldFormatterFixed
    extends LengthLimitingTextInputFormatter {
  final int? maxLength;
  LengthLimitingTextFieldFormatterFixed(this.maxLength) : super(maxLength);

  @override
  TextEditingValue formatEditUpdate(
    TextEditingValue oldValue,
    TextEditingValue newValue,
  ) {
    if (maxLength == null ||
        maxLength == -1 ||
        newValue.text.length <= maxLength!) {
      return newValue;
    }
    if (maxLength! > 0 && newValue.text.length > maxLength!) {
      // If already at the maximum and tried to enter even more, keep the old
      // value.
      if (oldValue.text.length == maxLength) {
        return oldValue;
      }

      // ignore: invalid_use_of_visible_for_testing_member
      return LengthLimitingTextInputFormatter.truncate(newValue, maxLength!);
    }
    return newValue;
  }
}

class TaxCodeFormatter extends TextInputFormatter {
  @override
  TextEditingValue formatEditUpdate(
      TextEditingValue oldValue, TextEditingValue newValue) {
    if (newValue.text.length == 11) {
      String newString;
      if (newValue.text.endsWith('-')) {
        newString = newValue.text;
      } else {
        newString = newValue.text.substring(0, 10) +
            '-' +
            newValue.text.substring(10, 11);
      }

      if (oldValue.text.length == 12) {
        newString = newString.substring(0, 10);
      }
      return TextEditingValue(
        text: newString,
        selection: TextSelection.collapsed(offset: newString.length),
      );
    }
    if (newValue.text.endsWith('-') && newValue.text.length != 11) {
      return oldValue;
    }
    return newValue;
  }
}

class NumericTextFormatter extends TextInputFormatter {
  //0: VND, 1: Foreign
  final int type;

  final bool isDot;

  NumericTextFormatter({
    this.type = 0,
    this.isDot = false,
  });

  @override
  TextEditingValue formatEditUpdate(
      TextEditingValue oldValue, TextEditingValue newValue) {
    if (newValue.text.isEmpty) {
      return newValue.copyWith(text: '');
    } else if (newValue.text.compareTo(oldValue.text) != 0) {
      if (newValue.text.length == 1 && newValue.text == '-') {
        return newValue;
      }
      if (newValue.text.replaceAll(RegExp(r"[-0-9.,]"), '').isNotEmpty ||
          newValue.text.endsWith('-')) {
        return newValue = oldValue;
      }
      final newString = type == 0
          ? _formatCurrency(
              _formatNumberCurrency(
                newValue.text,
                isDot,
              ),
              isDot,
            )
          : formatCurrencyForeign(
              newValue.text,
              isDot,
            );

      return TextEditingValue(
        text: newString,
        selection: TextSelection.collapsed(offset: newString.length),
      );
    } else {
      return newValue;
    }
  }

  String _formatCurrency(dynamic number, bool isDot) {
    if (number == 0) return "0";
    String numberFormat = number.toDouble().toStringAsFixed(0);
    if (!(numberFormat == '1000000000' || numberFormat == '-1000000000')) {
      if (numberFormat.startsWith('-') && numberFormat.length >= 10) {
        numberFormat = numberFormat.substring(0, 10);
      } else if (numberFormat.length > 9) {
        numberFormat = numberFormat.substring(0, 9);
      }
    }
    return NumberFormat.currency(
      locale: isDot ? 'vi_VN' : 'en_US',
      symbol: '',
      decimalDigits: 0,
    )
        .format(_formatNumberCurrency(
          numberFormat,
          isDot,
        ))
        .trim();
  }

  double _formatNumberCurrency(String text, bool isDot) {
    if (text.isEmpty || text == '-') return 0;
    return double.tryParse(text.replaceAll(isDot ? '.' : ',', '')) ?? 0;
  }

  String formatCurrencyForeign(dynamic number, bool isDot) {
    if (number == 0) return "0";
    String first = number.toString().substring(
        0,
        number.toString().contains(isDot ? ',' : '.')
            ? number.toString().lastIndexOf(isDot ? ',' : '.')
            : null);
    if (first.startsWith('-') && first.length >= 12) {
      first = first.substring(0, 12);
    } else if (first.length >= 12) {
      first = first.substring(0, 11);
    }
    String last = number.toString().substring(
        number.toString().contains(isDot ? ',' : '.')
            ? number.toString().lastIndexOf(isDot ? ',' : '.')
            : number.toString().length,
        number.toString().length);
    if (last.length >= 7) {
      last = last.substring(0, 6);
    }
    String result = NumberFormat.currency(
          locale: isDot ? 'vi_VN' : 'en_US',
          symbol: '',
          decimalDigits: 0,
        )
            .format(
              _formatNumberCurrency(
                first,
                isDot,
              ),
            )
            .trim() +
        last;
    return result;
  }
}
